// $Header: /home/anthony/tolkien/include/RCS/dpldbin.h,v 1.6 1996/06/19 02:19:53 anthony Exp $
//
//      TOLKIEN: Tool Kit for Genetics-based Applications
//
//      Anthony Yiu-Cheung Tang, 1992-93.
//      Department of Computer Science
//      The Chinese University of Hong Kong.
//
#if !defined( __DPLDBIN_H )
#define     __DPLDBIN_H

#include <hpldbin.h>
#include <tritstr.h>

//
//
//      Class BinDiploid provides facilities for experiments on
//      dominance and diploidy of binary chromosomes.
//
//

class BinDiploid : public BinIndividual
{
public:
    BinDiploid();
    BinDiploid(const BinDiploid &);
    BinDiploid(unsigned, bool = false, bool = false);
    BinDiploid(const TritString &, const TritString &, bool = false);

    BinDiploid* clone() const;

    int   length() const;
    void  printOn( ostream  & ) const;

    void  resize(unsigned);

    void operator=(const BinDiploid &);
    void operator=(const BinHaploid &);
    void operator=(const BitString &);

    virtual const TritString & hmlgs1() const;
    virtual const TritString & hmlgs2() const;

    void  mutate(double);
    void  randomize();
    void  setRand(unsigned);

    void  invert(int);
    void  set(int);
    void  clear(int);
    void  invert();
    void  set();
    void  clear();

    int   test(int) const;

    virtual BinDiploid* oddPtCrossover(const TGAObj *, int) const;
    virtual BinDiploid* evenPtCrossover(const TGAObj *, int) const;
    virtual BinDiploid* uniformCrossover(const TGAObj *, double) const;

    const BitString&    asBitString() const;
    double        asDouble() const;
    unsigned long asLong() const;
    double        asDouble(int,int) const;
    unsigned long asLong(int,int) const;

    bool isGrayCode() const;
    void isGrayCode(bool flag);

    double  distance(const TGAObj *) const;

    unsigned hammingDistance(const BinIndividual& src) const
    {
	return __hammingDistance(src.asBitString());
    }

    unsigned __hammingDistance(const BitString & bits) const
    {
	return expressed.__hammingDistance(bits);
    }

    TGAObj* onePtCrossover(const BinIndividual*) const;
    TGAObj* twoPtCrossover(const BinIndividual*) const;
    
    TGAObj* _onePtCrossover(const BinDiploid *) const;
    TGAObj* _onePtCrossover(const BinHaploid *) const;
    TGAObj* _twoPtCrossover(const BinDiploid *) const;
    TGAObj* _twoPtCrossover(const BinHaploid *) const;

    TGAObj* _oddPtCrossover(const BinHaploid *, int) const;
    TGAObj* _oddPtCrossover(const BinDiploid *, int) const;
    TGAObj* _evenPtCrossover(const BinHaploid *, int) const;
    TGAObj* _evenPtCrossover(const BinDiploid *, int) const;
    TGAObj* _uniformCrossover(const BinHaploid *, double) const;
    TGAObj* _uniformCrossover(const BinDiploid *, double) const;

protected:

    TritString  hmlg1, hmlg2;   // the homologous chromosomes
    BinHaploid  expressed;      // the expressed phenotype of this object

    virtual void                mapDominanceAt(unsigned uIndex);
    virtual void                setDominance();
    virtual void                normalize();

};

inline  BinDiploid::BinDiploid()
{
     hmlg1.mutateDC(0.5);
     hmlg2.mutateDC(0.5);
}


inline BinDiploid::BinDiploid(const BinDiploid & src) :
      hmlg1(src.hmlg1), hmlg2(src.hmlg2), expressed(src.expressed)
{
}

inline BinDiploid* BinDiploid::clone() const
{
     return new BinDiploid(*this);
}

inline  const TritString & BinDiploid::hmlgs1() const
{
	return hmlg1;
}

inline  const TritString & BinDiploid::hmlgs2() const
{
	return hmlg2;
}

inline  void BinDiploid::setRand(unsigned uI)
{
	hmlg1.setRand(uI);
	hmlg2.setRand(uI);
	mapDominanceAt(uI);
}

inline  int BinDiploid::length() const
{
        return expressed.length();
}

inline  const BitString& BinDiploid::asBitString() const
{
        return expressed.asBitString();
}

inline  double BinDiploid::asDouble() const
{
	return bin2double(expressed.asBitString(),0,expressed.length());
}

inline  unsigned long BinDiploid::asLong() const
{
	return bin2long(expressed.asBitString(),0,expressed.length());
}

inline  double BinDiploid::asDouble(int nFrom, int nLength) const
{
	return bin2double(expressed.asBitString(), nFrom, nLength);
}

inline  unsigned long  BinDiploid::asLong(int nFrom, int nLength) const
{
	return bin2long(expressed.asBitString(), nFrom, nLength);
}

inline void BinDiploid::mapDominanceAt(unsigned uIndex)
{
        if (hmlg1.valueAt(uIndex) >= hmlg2.valueAt(uIndex))
            expressed.assign(uIndex, abs(hmlg1.valueAt(uIndex)));
        else
            expressed.assign(uIndex, abs(hmlg2.valueAt(uIndex)));
}

inline  bool  BinDiploid::isGrayCode() const
{
        return expressed.isGrayCode();
}

inline  void  BinDiploid::isGrayCode(bool flag)
{
	expressed.isGrayCode(flag);
}

inline  double BinDiploid::distance(const TGAObj * pObj) const
{
        // should use RTTI to check pObj
        return __hammingDistance(((const BinIndividual *) pObj)->asBitString());
}

inline  int BinDiploid::test(int pos) const
{
	return expressed.test(pos);
}

#endif

