// $Header: /home/anthony/tolkien/include/RCS/hpldbin.h,v 1.7 1996/06/19 02:19:53 anthony Exp $
//
//      TOLKIEN: Tool Kit for Genetics-based Applications
//
//      Anthony Yiu-Cheung Tang, 1992-93.
//      Department of Computer Science
//      The Chinese University of Hong Kong.
//
//	Anthony Yiu-Cheung Tang, 1996.
//
#if !defined( __HPLDBIN_H )
#define     __HPLDBIN_H

#include <tbitstr.h>
#include <math.h>
#include <tgaobj.h>
#include <binind.h>

//
//
//      Class BinHaploid is an enhancement of
//      class BitString of libg++ for GA applications.
//      The genetic operators on the BinHaploid class
//      are optimized using the specific functions provided
//      by class BitString.
//
//

class BinGene;
class BinHaploid : public BinIndividual
{
public:

    friend class BinGene;

    BinHaploid();
    BinHaploid(unsigned, bool = false, bool = false);
    BinHaploid(const BitString & , bool = false);
    BinHaploid(const char *, bool = false);
    BinHaploid(const BinHaploid &);

    void  printOn( ostream  & ) const;

    BinHaploid* clone() const;

    void  randomize();
    void  mutate(double);

    virtual BinHaploid* oddPtCrossover(const TGAObj *, int) const;
    virtual BinHaploid* evenPtCrossover(const TGAObj *, int) const;
    virtual BinHaploid* uniformCrossover(const TGAObj *, double) const;

    virtual BinGene  operator[](unsigned);
    virtual unsigned operator[](unsigned) const;

    void operator=(const char  *);
    void operator=(const BinDiploid &);
    void operator=(const BinHaploid &);
    void operator=(const BitString &);
    virtual void operator=(unsigned long);

    int test(int pos) const; 

    virtual void assign(unsigned, unsigned);
    virtual void invert();
    virtual void invert(int);
    virtual void set();
    virtual void set(int);
    virtual void set(int, int);
    virtual void clear();
    virtual void clear(int);
    virtual void clear(int, int);
    virtual int  length() const;
    virtual void resize(unsigned);

    virtual double         asDouble(int,int) const;
    virtual unsigned long  asLong(int,int) const;
    virtual double         asDouble() const;
    virtual unsigned long  asLong() const;
    virtual const BitString &  asBitString() const;

    virtual bool isGrayCode() const;
    virtual void isGrayCode(bool);

    unsigned hammingDistance(const BinIndividual &src) const
    {
	return __hammingDistance(src.asBitString());
    }

    unsigned __hammingDistance(const BitString &) const;

    double   distance(const TGAObj * pObj) const;

    TGAObj* onePtCrossover(const BinIndividual*) const;
    TGAObj* twoPtCrossover(const BinIndividual*) const;

    TGAObj* _onePtCrossover(const BinDiploid *) const;
    TGAObj* _twoPtCrossover(const BinDiploid *) const;

    TGAObj* _oddPtCrossover(const BinHaploid *, int) const;
    TGAObj* _oddPtCrossover(const BinDiploid *, int) const;
    TGAObj* _evenPtCrossover(const BinHaploid *, int) const;
    TGAObj* _evenPtCrossover(const BinDiploid *, int) const;
    TGAObj* _uniformCrossover(const BinHaploid *, double) const;
    TGAObj* _uniformCrossover(const BinDiploid *, double) const;

protected:

    BitString   bits;
    bool        graycodeFlag;     // TRUE if the bits represent Gray Code

};

class BinGene
{
public:
        BinGene(BinHaploid & src, unsigned uIndex);
        BinGene(const BinGene& src);
        BinGene& operator=(unsigned uVal);
        operator unsigned();

private:
    BinHaploid&    ind;
    const unsigned uOffset;
};

inline  BinGene::BinGene(BinHaploid& src, unsigned uIndex) :
        ind(src), uOffset(uIndex)
{
}

inline  BinGene::BinGene(const BinGene& src) :
        ind(src.ind), uOffset(src.uOffset)
{
}

inline  BinGene&  BinGene::operator=(unsigned uVal)
{
        ind.assign(uOffset, uVal);
	return *this;
}

inline BinGene::operator unsigned()
{
	return ind.bits.test(uOffset);
}

inline  BinHaploid::BinHaploid() : graycodeFlag(false) {}

inline  BinHaploid::BinHaploid(const BinHaploid & src) :
		    BinIndividual(src),
                    graycodeFlag(src.graycodeFlag),
                    bits(src.bits)
{
}

inline  BinHaploid* BinHaploid::clone() const
{
	return new BinHaploid(*this);
}

inline  unsigned BinHaploid::operator[](unsigned uIndex) const
{
	return bits.test(uIndex);
}

inline	BinGene BinHaploid::operator[](unsigned uIndex)
{
	return BinGene(*this, uIndex);
}

inline  void BinHaploid::operator=(const char  *pString)
{
	bits = atoBitString(pString);
}

inline  void BinHaploid::operator=(const BitString & src)
{
        bits = src;
}

inline  void BinHaploid::assign(unsigned uIndex, unsigned uVal)
{
	if (uVal)
	    bits.set(uIndex);
	else
	    bits.clear(uIndex);
}

inline  void BinHaploid::invert()
{
	~bits;
}

inline  void BinHaploid::invert(int nIndex)
{
        bits.invert(nIndex);
}

inline  void BinHaploid::set()
{
        bits.set();
}

inline  void BinHaploid::set(int nIndex)
{
        bits.set(nIndex);
}

inline  void BinHaploid::set(int nFrom, int nTo)
{
        bits.set(nFrom, nTo);
}

inline  void BinHaploid::clear()
{
        bits.clear();
}

inline  void BinHaploid::clear(int nIndex)
{
        bits.clear(nIndex);
}

inline  void BinHaploid::clear(int nFrom, int nTo)
{
	bits.clear(nFrom, nTo);
}

inline  int BinHaploid::length() const
{
        return bits.length();
}

inline  void BinHaploid::resize(unsigned uNewLen)
{
	TBitString tbits(bits);
        tbits.resize(uNewLen);
        bits = tbits;
}

inline  const BitString & BinHaploid::asBitString() const
{
	return bits;
}

inline  double BinHaploid::asDouble() const
{
        return bin2double(bits,0,bits.length());
}

inline  unsigned long  BinHaploid::asLong() const
{
        return bin2long(bits,0,bits.length());
}

inline  bool  BinHaploid::isGrayCode() const
{
	return graycodeFlag;
}

inline  void  BinHaploid::isGrayCode(bool flag)
{
        graycodeFlag = flag;
}


inline double BinHaploid::distance(const TGAObj * pObj) const
{
    return __hammingDistance(((const BinIndividual *) pObj)->asBitString());
}

inline int BinHaploid::test(int pos) const
{
	return bits.test(pos);
}

#endif

