// $Header: /home/anthony/tolkien/include/RCS/popcrd.h,v 1.4 1996/06/19 02:19:53 anthony Exp $
//
//      TOLKIEN: Tool Kit for Genetics-based Applications
//
//      Anthony Yiu-Cheung Tang, 1992-93.
//      Department of Computer Science
//      The Chinese University of Hong Kong.
//
#if !defined( __POPCRD_H )
#define     __POPCRD_H

#include <popsmpl.h>
#include <ranksb.h>

template <class PhenotypeFunction, 
          class TDistance, class Collection = vector<TGAObj*> >
class   CrowdingPopulation : public SimplePopulation<PhenotypeFunction, Collection>
{
public:
        CrowdingPopulation() :
		uCrowdingFactor(2), uCrowdingSubPop(2),
                SimplePopulation<PhenotypeFunction, Collection>()
        {
        }

        CrowdingPopulation(const CrowdingPopulation& src) :
		uCrowdingFactor(src.crowdingFactor()), 
                uCrowdingSubPop(src.crowdingSubPop()),
                SimplePopulation<PhenotypeFunction, Collection>(src)
        {
        }

	~CrowdingPopulation() {}

        void crowdingFactor(long newval)
        {
            uCrowdingFactor = rangeCheck(newval,1L,5L,1L);   
        }

        long crowdingFactor() const     
        {
            return uCrowdingFactor;
        }

        void crowdingSubPop(long newval)
        {
            uCrowdingSubPop = newval;
        }

        long crowdingSubPop() const
        {
            return uCrowdingSubPop;
        }

        void replace(vector<TGAObj*>& newInds, bool reEvaluate )
        //
        //      replace old individuals with members in newInds using De Jong's
        //      crowding model
        //
        {
            //
            // if uCrowdingFactor or uCrowdingSubPop is zero or
            // newInds is larger than inds replaces individuals
            // in a random manner               
            //
            long i, index;

            if ( uCrowdingFactor == 0 ||                                  
                 uCrowdingSubPop == 0 ||
                 newInds.size() >= inds.size() ) {                          
                SimplePopulation<PhenotypeFunction, Collection>::replace(newInds, reEvaluate);
                return;
            }

            if ( reEvaluate ) {
                //
                // all individuals are evaluated
                //
                for (i = 0; i < newInds.size(); i++) {
                     index = select(newInds[i]);     
                     delete inds[index];
                     inds[index] = newInds[i];
                }                                                          

                for (i = 0; i < inds.size(); i++) 
                     evaluate(inds[i]);
            }
            else {                                             
                //                                             
                // only new individuals are evaluated
                //
                for (i = 0; i < newInds.size(); i++) {
                    evaluate(newInds[i]);
                    index = select(newInds[i]);     
                    delete inds[index];
                    inds[index] = newInds[i];
                }
            }
       }

protected:
        long uCrowdingFactor, uCrowdingSubPop;
        TDistance dist;

        long worstInd() const
        //
        //  select worst individual from a random subpopulation
        //  of size uCrowdingSubPop
        //
        //  this function assumes the size of the population is nonzero
        //
        {
            long worst, candidate, i;
            double worstStrength, val;

            worst = tRand(0, popSize() - 1);
            worstStrength = inds[worst]->fitness();
        
            for (i = crowdingSubPop() - 1; i >= 0; i--) {
                 candidate = tRand(0, inds.size() - 1);
                 val = inds[candidate]->fitness();
                 if (worstStrength > val) {                
                     worst = candidate;       
                     worstStrength = val;
                 }                                                        
            }
             
            return worst;
        }

        long select(const TGAObj *pChild) const
        //                         
        //      return an index to the individual to be replaced
        //      using modified De Jong crowding     
        //
        {
            double match, matchMax = -1;     
            long member, mostSimilar, j;

            for (j = uCrowdingFactor - 1; j >=0 ; j--) {
                member = worstInd();
                // match = fabs(pChild->distance(inds[member]));
                match = fabs(dist(pChild,inds[member]));
                if (match > matchMax) {
                    matchMax = match;
                    mostSimilar = member;
                }
            }

            return mostSimilar;
        }
};

#endif

