// $Header: /home/anthony/tolkien/include/RCS/popelite.h,v 1.4 1996/06/19 02:19:53 anthony Exp $
//
//      TOLKIEN: Tool Kit for Genetics-based Applications
//
//      Anthony Yiu-Cheung Tang, 1992-93.
//      Department of Computer Science
//      The Chinese University of Hong Kong.
//
//      Anthony Yiu-Cheung Tang, 1996.
//
#if !defined(__POPELITE_H)
#define __POPELITE_H

#include <algo.h>
#include <populatn.h>

template < class Collection = vector<TGAObj*> >
class ElitePopulation : public Population<PhenotypeFunction, Collection> 
{
public:
        ElitePopulation(const ElitePopulation & src) 
        {
            // since the population is sorted
            // a new individual is added to the population
            // first, and the then least fit individual
            // is removed from the population
            // so reserve some more space
            inds.reserve(popSize() + 1);
            addAll(src.inds);
        }

        ElitePopulation(vector<TGAObj*> initPop) :
              Population<PhenotypeFunction, Collection>()
        {
              inds.reserve(initPop.size() + 1);
              for (vector<TGAObj*>::iterator i = initPop.begin();
                   i < initPop.end(); ++i) {
                   evaluate(*i);
                   inds.push_back(*i);
              }
	      sort(inds.begin(),inds.end(),DecFitness());
        }

	~ElitePopulation() { }

        void add(TGAObj* pObj, bool bEval = true)
        {
             if (bEval)
                evaluate(pObj);
             Collection::iterator i = 
             lower_bound(inds.begin(), inds.end(), pObj, DecFitness());
             inds.insert(i, pObj);

             // now remove the least fit 
             i = inds.end();
             --i;
             delete (*i);
             inds.erase(i);
        }

        void  replace(vector<TGAObj*>& newInds, bool reEvaluate) 
        //
        //      replace old individuals with members in newInds
        //      individuals with the least fitness is replaced first
        //
        //
        {
            long popsize = popSize();
            vector<TGAObj*>::iterator i;

            if ( reEvaluate ) {
                //
                // evaluate all individuals
                //
                vector<TGAObj*> vec = inds;
                for (i = newInds.begin(); i < newInds.end(); ++i) 
                     vec.push_back(*i);
                // vec now contains the original population
                // and the new individuals
                for (i = vec.begin(); i < vec.end(); ++i)
		     evaluate(*i);
	        sort(vec.begin(),vec.end(),DecFitness());

                // empty inds
                inds.erase(inds.begin(), inds.end());

                // remove the least fit individuals
                for (i = vec.begin() + popsize; i < vec.end(); ++i)
                    delete *i;
                vec.erase(vec.begin() + popsize, vec.end());
                
                // move the individuals back to inds
                inds = vec;
            }
            else 
                for (i = newInds.begin(); i < newInds.end(); ++i) 
                     add(*i, true);
	}
};

#endif

