// $Header: /home/anthony/tolkien/include/RCS/slctlrnk.h,v 1.4 1996/06/19 02:19:53 anthony Exp $
//
//      TOLKIEN: Tool Kit for Genetics-based Applications
//
//      Anthony Yiu-Cheung Tang, 1992-93.
//      Department of Computer Science
//      The Chinese University of Hong Kong.
//
//      Anthony Yiu-Cheung Tang, 1996.
//
#if !defined( __SLCTLRNK_H)
#define __SLCTLRNK_H

#include <populatn.h>
#include <algo.h>

template < class Collection = vector<TGAObj*> >
class  LinearRanking : public SelectionScheme<Collection>
{
//
//      Linear ranking Selection Scheme as described in
//      ICGA-91 : The GENITOR Algorithm and Selection Pressure :
//                Why Rank-Based Allocation of Reproductive Trials is Best
//
//
public:
	const float DEFAULT_BIAS = 1.5;

        LinearRanking(double val=DEFAULT_BIAS) : 
              SelectionScheme<Collection>()
        {
              bias(val);
        }

	const TGAObj* select()
        //
        //      the population to be selected is first sorted in decending order
        //      of fitness
        //
        //      a selection index is generated by
        //
        //
        //      index = (population_size / (2 * (bias - 1))) *
        //               (bias - sqrt(bias*bias - 4(bias - 1) * rnd(0,1)))
        //
        {
              int index;

              index = (int) (sorted.size() *
                 (flBias -
                 sqrt(flBiasSqr - tRand.asDouble() * 4 * flBiasLessOne)) /
                 (2 * flBiasLessOne));

              count();

              return sorted[index];
        }

	void init()
        {
	     sorted.erase(sorted.begin(),sorted.end());
             sorted.reserve(pPop->popSize());
             sorted.insert(sorted.begin(), pPop->begin(), pPop->end());
             sort(sorted.begin(),sorted.end(),DecFitness());
        }

        double bias() const
        {
	     return flBias;
        }

        virtual double bias(double newBias)
        {
	    flBias = newBias > 1 ? newBias : DEFAULT_BIAS;
	    flBiasSqr = flBias * flBias;
	    flBiasLessOne = flBias - 1;
        }

protected:
	double flBias, flBiasSqr, flBiasLessOne;
	vector<TGAObj*> sorted;
};


#endif

