// $Header: /home/anthony/tolkien/include/RCS/tgaobj.h,v 1.8 1996/06/19 02:19:53 anthony Exp $
//
//      TOLKIEN: Tool Kit for Genetics-based Applications
//
//      Anthony Yiu-Cheung Tang, 1992-93.
//      Department of Computer Science
//      The Chinese University of Hong Kong.
//
//      Anthony Yiu-Cheung Tang, 1996.
//
#if !defined( __TOBJECT_H )
#define __TOBJECT_H 

#include <tdefs.h>
#include <string.h>
#include <iostream.h>
#include <assert.h>
#include <values.h>
#include <builtin.h>
#include <trand.h>

//
// class TGAObj is the base class for the data structures manipulated
// by a genetic algorithm.
//
class TGAObj 
{
public :

    virtual ~TGAObj() {}

    virtual TGAObj* clone() const = 0; // deep copy

    virtual void printOn(ostream& out = cout) const;

    virtual hashValue_t hashValue() const;

    double fitness() const
    {
	return flFitness;
    }

    double objValue() const
    {
	return flObjValue;
    }

    virtual void fitness(double val)
    {
	flFitness = val;
    }

    virtual void objValue(double val)
    {
	flObjValue = val;
    }

    bool  isSame(const TGAObj& obj) const
    {
	return this == &obj;
    }

    virtual int	length() const = 0;

    virtual void     	mutate(double) = 0;
    virtual void     	randomize() = 0;

    friend ostream & operator << ( ostream &, const TGAObj &);

protected :

    TGAObj() : flFitness(0), flObjValue(0) 
    {
    }

private :
    double flFitness;   // scaled fitness, used for selection
    double flObjValue;  // raw fitness, returned by the fitness function
};

inline void TGAObj::printOn(ostream& out) const
{
	out << objValue() << ':' << fitness();
}

inline ostream & operator << ( ostream & out, const TGAObj & obj )
{
    obj.printOn( out );
    return out;
}

//
// the don't care symbol 
//
static struct DontCare
{
public:
   operator char() const
   {
	return '#';
   }
   operator int()  const
   {
	return -1;
   }
} dontCare;

//
// STL Function Objects used for selection, scaling, ... etc
//
class DecFitness
{
public :
	bool operator()(TGAObj* x, TGAObj* y) const
        {
	    return x->fitness() > y->fitness();
	}
};

class AscFitness
{
public :
	bool operator()(TGAObj* x, TGAObj* y) const
        {
	    return x->fitness() < y->fitness();
	}
};

class DecObjValue
{
public :
	bool operator()(TGAObj* x, TGAObj* y) const
        {
	    return x->objValue() > y->objValue();
	}
};

class AscObjValue
{
public :
	bool operator()(TGAObj* x, TGAObj* y) const
        {
	    return x->objValue() < y->objValue();
	}
};

// if nValue is not between nMin and nMax (inclusive)
//    return nDefval
// else
//    return nValue

template <class T>
T rangeCheck(T value, T min, T max, T defVal)
{
     return ((value < min) ?
		defVal : (value > max) ? defVal : value);
}

#endif

